"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
require("reflect-metadata");
const utils_1 = require("power-di/utils");
const routes = [];
const methods = ['get', 'put', 'post', 'delete', 'patch'];
function getNameAndMethod(functionName) {
    let name = functionName, functionMethod = 'get';
    functionName = functionName.toLowerCase();
    for (let i = 0; i < methods.length; i++) {
        const method = methods[i];
        if (functionName.startsWith(method)) {
            name = functionName.substring(3);
            functionMethod = method;
            break;
        }
    }
    return { name, method: functionMethod };
}
const COMMENTS = /((\/\/.*$)|(\/\*[\s\S]*?\*\/))/mg;
const DEFAULT_PARAMS = /=[^,]+/mg;
const FAT_ARROWS = /=>.*$/mg;
// TS_TO_ES6
const ASYNC_DEFAULT_PARAMS = ', void 0, void 0, function* (';
function getParameterNames(fn) {
    let code = fn.toString()
        .replace(ASYNC_DEFAULT_PARAMS, '')
        .replace(COMMENTS, '')
        .replace(FAT_ARROWS, '')
        .replace(DEFAULT_PARAMS, '');
    let result = code
        .slice(code.indexOf('(') + 1, code.indexOf(')'))
        .match(/([^\s,]+)/g);
    return result === null
        ? []
        : result;
}
const extRules = {};
const getRuleKey = (target, key) => `${utils_1.getGlobalType(target.constructor)}_${key}`;
function getMethodRules(target, key) {
    const ruleKey = getRuleKey(target, key);
    if (!extRules[ruleKey]) {
        extRules[ruleKey] = {
            param: {},
        };
    }
    return extRules[ruleKey];
}
// #region ParameterDecorator
function FromCustom(custom, paramName) {
    return (target, key, index) => {
        const methodRule = getMethodRules(target, key);
        methodRule.param[paramName || index] = custom;
    };
}
exports.FromCustom = FromCustom;
function FromBody(paramName) {
    return FromCustom((ctx, name) => ctx.request.body[name], paramName);
}
exports.FromBody = FromBody;
function FromParam(paramName) {
    return FromCustom((ctx, name) => ctx.params[name], paramName);
}
exports.FromParam = FromParam;
function FromQuery(paramName) {
    return FromCustom((ctx, name) => ctx.query[name], paramName);
}
exports.FromQuery = FromQuery;
// #endregion
// #region Controller Middleware
const beforeMiddlewares = {};
const afterMiddlewares = {};
function beforeMiddware(middwares) {
    middwares = [].concat(middwares);
    return (target, key) => {
        const typeGlobalName = utils_1.getGlobalType(target.constructor);
        const route = routes.find(r => r.typeGlobalName === typeGlobalName && r.functionName === key);
        if (route) {
            route.beforeMiddleware.concat(middwares);
        }
        else {
            beforeMiddlewares[getRuleKey(target, key)] = middwares;
        }
    };
}
exports.beforeMiddware = beforeMiddware;
function afterMiddware(middwares) {
    middwares = [].concat(middwares);
    return (target, key) => {
        const typeGlobalName = utils_1.getGlobalType(target.constructor);
        const route = routes.find(r => r.typeGlobalName === typeGlobalName && r.functionName === key);
        if (route) {
            route.afterMiddleware.concat(middwares);
        }
        else {
            afterMiddlewares[getRuleKey(target, key)] = middwares;
        }
    };
}
exports.afterMiddware = afterMiddware;
// #endregion
function routerMetadata(data = {}) {
    return function (target, key) {
        const typeGlobalName = utils_1.getGlobalType(target.constructor);
        const CtrlType = target.constructor;
        const routerFn = target[key];
        const paramTypes = Reflect.getMetadata('design:paramtypes', target, key) || [];
        const typeInfo = Object.assign({}, data, { typeGlobalName, typeClass: CtrlType, functionName: key, paramTypes: getParameterNames(routerFn).map((name, i) => {
                return {
                    name,
                    type: paramTypes[i],
                };
            }), returnType: Reflect.getMetadata('design:returntype', target, key), beforeMiddleware: beforeMiddlewares[getRuleKey(target, key)] || [], afterMiddleware: afterMiddlewares[getRuleKey(target, key)] || [], call: () => () => { } });
        let name = getNameAndMethod(typeInfo.functionName);
        if (!typeInfo.url) {
            const ctrl = typeGlobalName
                .split('_')[0]
                .toLowerCase()
                .replace('controller', '');
            typeInfo.url = `/${ctrl}/${name.name}`;
        }
        if (!typeInfo.method) {
            typeInfo.method = name.method;
        }
        routes.push(typeInfo);
        const methodRules = getMethodRules(target, key);
        const getArgs = (ctx) => {
            return typeInfo.paramTypes.map((p, i) => {
                const name = p.name;
                let argValue;
                if (methodRules.param[i]) {
                    argValue = methodRules.param[i](ctx, name);
                }
                else {
                    const param = ctx.params || {};
                    const query = ctx.query || {};
                    const body = (ctx.request || {}).body || {};
                    argValue = param[name] || query[name] || body[name] || undefined;
                }
                if (argValue === undefined) {
                    return argValue;
                }
                switch (p.type) {
                    case Number:
                        return parseFloat(argValue);
                    case String:
                        return `${argValue}`;
                    // TODO custom TypeFormatter
                    default:
                        return argValue;
                }
            });
        };
        const call = function (ctx) {
            return __awaiter(this, void 0, void 0, function* () {
                const context = this || ctx;
                const ctrl = new CtrlType(context);
                const args = getArgs(context);
                try {
                    const ret = yield Promise.resolve(routerFn.apply(ctrl, args));
                    if (ret !== undefined) {
                        context.body = ret;
                    }
                    return ret;
                }
                catch (error) {
                    this.throw(400, error);
                }
            });
        };
        typeInfo.call = () => target[key];
        return {
            value: call
        };
    };
}
exports.routerMetadata = routerMetadata;
function getRouters() {
    return routes;
}
exports.getRouters = getRouters;
